var fs = require('fs'),
    path = require('path');

var log = require('color-log'),
    _ = require('underscore');

var config = require('./config');

var currentPath = process.cwd(),
    masterModules = [],
    modulesDepend = {},
    rootModulePath = '',
    fileExt = '.js',
    comboFileExt = '-combo';

var comboFinishCallback = null;

exports.dependencies = function(modules, options, comboFinish) {

    fileExt = options.debug ? '-' + options.debug + fileExt : fileExt;
    comboFinishCallback = comboFinish;
    masterModules = Array.isArray(modules) ? modules : [modules];

    if (masterModules.length === 0) {

        log.warn('需要搜索的modules为空');
        process.exit(1);
    }

    config.getConfig(function(data) {

        rootModulePath = path.join(currentPath, data.moduleSrc);

        masterModules.forEach(function(v, k) {

            findModuleDepend(v, [v], function(masterModule, dependencies) {
                modulesDepend[masterModule] = _.extend(modulesDepend[masterModule] || {}, dependencies);
            }, findModuleDependFinish);
        });
    });
}

function findModuleDepend(masterModule, modules, callback, finish) {

    fs.readdir(rootModulePath, function(err, files) {
        if (err) {
            log.error(moduleName + ': 模块不存在');
            process.exit(1);
        } else {
            (function next(i) {

                if (i < modules.length) {

                    if (_.indexOf(files, modules[i]) === -1) {

                        log.error(modules[i] + ': 模块不存在');
                        process.exit(1);
                    }

                    var moduleDir = path.join(rootModulePath, modules[i]);
                    fs.readdir(moduleDir, function(err, files) {

                        files = files.sort().reverse();
                        var packagePath = path.join(moduleDir, files[0], 'package.json');

                        fs.readFile(packagePath, function(err, data) {
                            if (err) {
                                log.error(modules[i] + ': package.json文件读取失败');
                                process.exit(1);
                            } else {

                                var dependencies = JSON.parse(data.toString()).dependencies;
                                if (_.isEmpty(dependencies)) {
                                    next(i + 1);
                                } else {

                                    var keys = Object.keys(dependencies);
                                    callback(masterModule, dependencies);

                                    findModuleDepend(masterModule, keys, callback, function() {
                                        next(i + 1);
                                    });
                                }
                            }
                        });
                    });
                } else {
                    finish && finish(masterModule);
                }
            }(0));
        }
    });
}

function findModuleDependFinish(masterModule) {

    var modulePath = path.join(rootModulePath, masterModule);
    fs.readdir(modulePath, function(err, files) {

        files = files.sort();
        var moduleFile = path.join(modulePath, files[0], masterModule + fileExt);
        var comboFile = path.join(modulePath, files[0], masterModule + comboFileExt + fileExt);

        fs.readFile(moduleFile, function(err, data) {

            if (err) {
                log.error('findModuleDependFinish: ' + masterModule + '模块读取失败')
            } else {
                fs.writeFile(comboFile, data, function(err) {
                    if (err) {
                        log.error(moduleFile + ': 主文件copy失败');
                        process.exit(1);
                    }
                    comboModeulDepend(masterModule, comboFile);
                });
            }
        });
    });
}

function comboModeulDepend(masterModule, comboFile) {

    (function next(i) {

        var keys = Object.keys(modulesDepend[masterModule]);
        if (i < keys.length) {

            var modulePath = path.join(rootModulePath, keys[i]);
            fs.readdir(modulePath, function(err, files) {

                files = files.sort();
                var moduleFile = path.join(modulePath, files[0], keys[i] + fileExt);

                fs.readFile(moduleFile, function(err, data) {

                    if (err) {
                        log.error('依赖模块读取失败');
                        process.exit(1);
                    } else {
                        fs.appendFile(comboFile, data, function(err) {
                            if (err) {
                                log.error('模块合并失败：' + err.message);
                                process.exit(1);
                            }
                            next(i + 1);
                        });
                    }
                });
            });
        } else {
            comboFinishCallback && comboFinishCallback(masterModule);
        }
    }(0));
}